@@ -43,6 +43,8 @@ import pytest
 import py
 import time
 import urllib2
+import httplib
+import ConfigParser
 
 from selenium.webdriver.firefox.firefox_binary import FirefoxBinary
 from selenium import selenium
@@ -101,7 +103,7 @@ def pytest_runtest_setup(item):
 
 
 def pytest_runtest_teardown(item):
-    if TestSetup.selenium and not 'skip_selenium' in item.keywords:
+    if hasattr(TestSetup, 'selenium') and TestSetup.selenium and not 'skip_selenium' in item.keywords:
         _capture_debug(item)
         _stop_selenium(item)
 
@@ -111,15 +113,22 @@ def pytest_funcarg__mozwebqa(request):
 
 
 def pytest_addoption(parser):
+    config = ConfigParser.ConfigParser(defaults={
+        'baseurl': '',
+        'api': 'webdriver'
+    })
+    config.read('mozwebqa.cfg')
+
     group = parser.getgroup('selenium', 'selenium')
     group._addoption('--baseurl',
                      action='store',
                      dest='base_url',
+                     default=config.get('DEFAULT', 'baseurl'),
                      metavar='url',
                      help='base url for the application under test.')
     group._addoption('--api',
                      action='store',
-                     default='webdriver',
+                     default=config.get('DEFAULT', 'api'),
                      metavar='api',
                      help="version of selenium api to use. 'rc' uses selenium rc. 'webdriver' uses selenium webdriver (the default).")
     group._addoption('--host',
@@ -202,8 +211,8 @@ def pytest_addoption(parser):
                     action='store',
                     dest='webqa_report_path',
                     metavar='path',
-                    default='results.html',
-                    help="create mozilla webqa custom report file at given path. default is 'results.html'")
+                    default='results/index.html',
+                    help="create mozilla webqa custom report file at given path. default is 'results/index.html'")
 
 
 def _get_status_code(url):
@@ -270,6 +279,16 @@ def _check_selenium_usage(item):
             raise pytest.UsageError("--browser or --environment must be specified when using the 'rc' api.")
 
 
+def _get_common_sauce_settings(item):
+    config = ConfigParser.ConfigParser(defaults={'tags': ''})
+    config.read('mozwebqa.cfg')
+    tags = config.get('DEFAULT', 'tags').split(',')
+    tags.extend([mark for mark in item.keywords.keys() if not mark.startswith('test')])
+    return {'name': ".".join(_split_class_and_test_names(item.nodeid)),
+            'tags': tags,
+            'public': False}
+
+
 def _start_selenium(item):
     if item.api == 'webdriver':
         _start_webdriver_client(item)
@@ -279,13 +298,10 @@ def _start_selenium(item):
 
 def _start_webdriver_client(item):
     if item.sauce_labs_credentials_file:
-        capabilities = {
-                    'platform': item.platform,
-                    'browserName': item.browser_name,
-                    'version': item.browser_version,
-                    'name': ".".join(_split_class_and_test_names(item.nodeid)),
-                    'tags': item.keywords.keys()[:-1],
-                    'public': False}
+        capabilities = _get_common_sauce_settings(item)
+        capabilities.update({'platform': item.platform,
+                         'browserName': item.browser_name,
+                         'version': item.browser_version})
         executor = 'http://%s:%s@ondemand.saucelabs.com:80/wd/hub' % (item.sauce_labs_credentials['username'], item.sauce_labs_credentials['api-key'])
         TestSetup.selenium = webdriver.Remote(command_executor=executor,
                                               desired_capabilities=capabilities)
@@ -324,18 +340,15 @@ def _start_webdriver_client(item):
 
 
 def _start_rc_client(item):
-    test_name = ".".join(_split_class_and_test_names(item.nodeid))
     if item.sauce_labs_credentials_file:
+        settings = _get_common_sauce_settings(item)
+        settings.update({'username': item.sauce_labs_credentials['username'],
+                         'access-key': item.sauce_labs_credentials['api-key'],
+                         'os': item.platform,
+                         'browser': item.browser_name,
+                         'browser-version': item.browser_version})
         TestSetup.selenium = selenium('ondemand.saucelabs.com', '80',
-                                      json.dumps({
-                                      'username': item.sauce_labs_credentials['username'],
-                                      'access-key': item.sauce_labs_credentials['api-key'],
-                                      'os': item.platform,
-                                      'browser': item.browser_name,
-                                      'browser-version': item.browser_version,
-                                      'name': test_name,
-                                      'tags': item.keywords.keys()[:-1],
-                                      'public': False}),
+                                      json.dumps(settings),
                                       TestSetup.base_url)
     else:
         browser = item.environment or item.browser
@@ -350,7 +363,7 @@ def _start_rc_client(item):
         _capture_session_id(item, _debug_path(item))
 
     TestSetup.selenium.set_timeout(TestSetup.timeout)
-    TestSetup.selenium.set_context(test_name)
+    TestSetup.selenium.set_context(".".join(_split_class_and_test_names(item.nodeid)))
 
 
 def _debug_path(item):
@@ -459,16 +472,12 @@ class LogHTML(object):
 
     def _appendrow(self, result, report):
         (classname, testname) = _split_class_and_test_names(report.nodeid)
-        filename = os.path.sep.join(["debug", _generate_filename(classname, testname)])
         time = self._durations.pop(report.nodeid, 0.0)
-        try:
-            session_id = open('%s.session' % filename, 'r').readline()
-        except:
-            session_id = None
-        links = {'HTML': '%s.html' % filename,
-                 'Screenshot': '%s.png' % filename}
+        session_id = self._get_session_id(report)
+        links = {'HTML': self._get_debug_filename(report, 'html'),
+                 'Screenshot': self._get_debug_filename(report, 'png')}
         if self.config.option.capture_network:
-            links['Network'] = '%s.json' % filename
+            links['Network'] = self._get_debug_filename(report, 'json')
         if session_id:
             links['Sauce Labs Job'] = 'http://saucelabs.com/jobs/%s' % session_id
         # Log may contain passwords, etc so we shouldn't capture this until we can do so safely
@@ -532,6 +541,42 @@ class LogHTML(object):
                 self.test_logs.append('\n</object></div>')
             self.test_logs.append('\n</td></tr>')
 
+    def _get_debug_filename(self, report, extension):
+        filename = os.path.join('debug', _generate_filename(*_split_class_and_test_names(report.nodeid)))
+        return '%s.%s' % (filename, extension)
+
+    def _get_session_id(self, report):
+        report_dir = self._make_report_dir()
+        try:
+            session_id = open(os.path.join(report_dir, self._get_debug_filename(report, 'session')), 'r').readline()
+        except:
+            session_id = None
+        return session_id
+
+    def _make_report_dir(self):
+        logfile_dirname = os.path.dirname(self.logfile)
+        if logfile_dirname and not os.path.exists(logfile_dirname):
+            os.makedirs(logfile_dirname)
+        return logfile_dirname
+
+    def _send_result_to_sauce(self, report):
+        session_id = self._get_session_id(report)
+
+        if session_id:
+            try:
+                result = {'passed': report.passed or (report.failed and 'xfail' in report.keywords)}
+                credentials = _credentials(self.config.option.sauce_labs_credentials_file)
+                basic_authentication = ('%s:%s' % (credentials['username'], credentials['api-key'])).encode('base64')[:-1]
+                connection = httplib.HTTPConnection('saucelabs.com')
+                connection.request('PUT', '/rest/v1/%s/jobs/%s' % (credentials['username'], session_id),
+                                   json.dumps(result),
+                                   headers = {
+                                       'Authorization': 'Basic %s' % basic_authentication,
+                                       'Content-Type': 'text/json'})
+                connection.getresponse()
+            except:
+                pass
+
     def append_pass(self, report):
         self.passed += 1
         self._appendrow('Passed', report)
@@ -557,6 +602,9 @@ class LogHTML(object):
             self.skipped += 1
 
     def pytest_runtest_logreport(self, report):
+        if self.config.option.sauce_labs_credentials_file:
+            self._send_result_to_sauce(report)
+
         if report.passed:
             self.append_pass(report)
         elif report.failed:
@@ -578,9 +626,7 @@ class LogHTML(object):
         self.suite_start_time = time.time()
 
     def pytest_sessionfinish(self, session, exitstatus, __multicall__):
-        logfile_dirname = os.path.dirname(self.logfile)
-        if logfile_dirname and not os.path.exists(logfile_dirname):
-            os.makedirs(logfile_dirname)
+        self._make_report_dir()
         logfile = py.std.codecs.open(self.logfile, 'w', encoding='utf-8')
 
         suite_stop_time = time.time()
@@ -617,7 +663,7 @@ class LogHTML(object):
             selenium_server = self.config.option.sauce_labs_credentials_file and 'Sauce Labs' or 'http://%s:%s' % (self.config.option.host, self.config.option.port)
             logfile.write('\n<tr><th>Selenium Server</th><td>%s</td></tr>' % selenium_server)
             if self.config.option.api.upper() == 'WEBDRIVER' or self.config.option.sauce_labs_credentials_file:
-                logfile.write('\n<tr><th>Browser</th><td>%s %s on %s</td></tr>' % (self.config.option.browser_name.title(), self.config.option.browser_version, self.config.option.platform.title()))
+                logfile.write('\n<tr><th>Browser</th><td>%s %s on %s</td></tr>' % (str(self.config.option.browser_name).title(), self.config.option.browser_version, str(self.config.option.platform).title()))
             else:
                 logfile.write('\n<tr><th>Browser</th><td>%s</td></tr>' % self.config.option.environment or self.config.option.browser)
             if self.config.option.api.upper() == 'RC':
